﻿<?php
///////////////////////////////////////////////////////////////////////////

require_once 'lib/hashed_array.php';
require_once 'lib/tv/abstract_tv.php';
require_once 'lib/tv/default_epg_item.php';
require_once 'lib/utils.php';

require_once 'demo_channel.php';

///////////////////////////////////////////////////////////////////////////

class DemoAresTv extends AbstractTv
{
    private static $arescam_exe;
    ///////////////////////////////////////////////////////////////////////

    public function __construct()
    {
        DemoAresTv::$arescam_exe = dirname(__FILE__) . '/bin/arescam';

        parent::__construct(
            AbstractTv::MODE_CHANNELS_N_TO_M,
            DemoConfig::TV_FAVORITES_SUPPORTED,
            true);
    }

    public function run_arescrypt (&$plugin_cookies) {
        // Here we should insert arescrypt.ko and start arescam-daemon.
        //
        // The thing is that we might already have the daemon running and
        // the kernel module inserted (that might happen if PHP engine was
        // restarted in the middle).
        //
        // The logic will be as follows:
        //   - kill arescam (if it is running):
        //     - check PID file;
        //     - remove PID file;
        //   - remove arescrypt.ko;
        //   - detect smp family;
        //   - insert arescrypt.ko;
        //   - start up arescam-daemon;
        //   - check new PID file.

        // 1. Kill arescam.

        hd_print ("--> Reading PID-file '" . DemoConfig::daemon_pid_file . "'...");
        hd_silence_warnings ();
        $pid = file_get_contents (DemoConfig::daemon_pid_file);
        hd_restore_warnings ();
        if ($pid !== false) {
            hd_print ("--> PID: $pid");
            $cmd = sprintf ('kill %s', escapeshellarg ($pid));
            hd_print ("cmd_kill: $cmd");
            system ($cmd);
            hd_print ("--> unlinking " . DemoConfig::daemon_pid_file);
            unlink (DemoConfig::daemon_pid_file);
        }

        // 2. rmmod arescrypt.ko

        $cmd = sprintf ('rmmod arescrypt');
        hd_print("cmd_rmmod: $cmd");
        system ($cmd);

        // 3. detect smp family and arescrypt kernel module
        $kernel_release = system ('uname -r', $rc);
        if ($rc !== 0) {
            hd_print ("Error: uname -r failed");
            throw_ares_error ();
        }
        $cmd = sprintf (DemoConfig::cmd_smp_family);
        //hd_print ("cmd_smp_family: $cmd");
        $smp_family = system ($cmd, $rc);
        if ($smp_family === false) {
            hd_print ("Error: can not detect smp family.");
            throw_ares_error ();
        }
        $smp_family = strtolower (chop ($smp_family));
        hd_print ("smp family: $smp_family");
        $arescrypt_ko = dirname(__FILE__) . '/modules/' .
                        $kernel_release . '/' . $smp_family . '/arescrypt.ko';
        if ($smp_family == "867x") {
            $cmd = sprintf (DemoConfig::cmd_kernel_chksum);
            $kernel_chksum = system ($cmd, $rc);
            hd_print ("kernel config chksum: $kernel_chksum");
            $chksum_parts = explode (" ", $kernel_chksum);
            $kernel_chksum2 = intval ($chksum_parts [1]);
            if ($kernel_chksum2 >= 38900/*$kernel_chksum == "3527659131 38900"*/) {
                // Dune Kernel SDK 2012_07_13 (новый, публичные прошивки 120531_2200_beta и новее)
                $sdk_version = "sdk20120713";
                $arescrypt_ko = dirname(__FILE__) . '/modules/' .
                                $kernel_release . '/' . $smp_family . '/' .
                                $sdk_version . '/arescrypt.ko';
            } else if ($kernel_chksum == "3322656547 36120") {
                // Dune Kernel SDK 2011_10_19 (предыдущий, публичные прошивки 120329_2012 и старше)
            } else if ($kernel_chksum == "915029021 38163") {
                // Dune Kernel SDK 2012_12_28 (Linux kernel:  2.6.32, Dune HD TV-102, Dune HD Connect)
            }
        }
        hd_print ("arescrypt_ko: $arescrypt_ko");

        // 4. insmod arescrypt.ko

        $cmd = sprintf ('insmod %s', escapeshellarg ($arescrypt_ko));
        hd_print ("cmd_insmod: $cmd");
        system ($cmd, $rc);
        if ($rc !== 0) {
            hd_print ("Error: insmod failed");
            throw_ares_error ();
        }

        // 5. start arescam daemon

        $cmd = sprintf (DemoConfig::cmd_daemon, escapeshellcmd (DemoAresTv::$arescam_exe),
                                           escapeshellarg (DemoConfig::auth_url),
                                           escapeshellarg (DemoConfig::daemon_pid_file));

        $use_proxy = isset ($plugin_cookies->use_proxy) ? $plugin_cookies->use_proxy : 'no';
        if ($use_proxy == 'yes') {
            $proxy_ip = $plugin_cookies->proxy_ip;
            $proxy_port = $plugin_cookies->proxy_port;
            $cmd = $cmd . ' -y http://' . $proxy_ip . ':' . $proxy_port;
        }
        hd_print ("cmd_daemon: $cmd");
        system ($cmd, $rc);
        if ($rc !== 0) {
            hd_print("Error: daemon failed to start");
            throw_ares_error();
        }

        // 6. check PID file

        hd_print ("--> Checking PID-file '" . DemoConfig::daemon_pid_file . "'...");
        for ($i = 0; $i < 5; ++$i) {
            hd_silence_warnings ();
            $pid = file_get_contents (DemoConfig::daemon_pid_file);
            hd_restore_warnings ();
            if ($pid !== false) {
                hd_print ("--> Daemon started; pid: $pid");
                break;
            }
            hd_print ('--> No PID-file. Waiting...');
            usleep (100000);
        }
        if ($pid === false) // XXX: that might be actually an error...
            hd_print ("Warning: no PID-file within 0.5 sec after start.");
    }

    public function get_fav_icon_url()
    {
        return DemoConfig::FAV_CHANNEL_GROUP_ICON_PATH;
    }

    ///////////////////////////////////////////////////////////////////////

    protected function load_channels(&$plugin_cookies)
    {


        $res = $this->login_terminal (null);
        if ($res) {
            $this->run_arescrypt ($plugin_cookies);
        }

        $json_reply = $this->api_call (
            HD::make_json_rpc_request ('get_playlists', array (
                'macaddr' => str_replace (':', '-', HD::get_mac_addr ())
            ))
        );

        $this->channels = new HashedArray();
        $this->groups = new HashedArray();

        if ($this->is_favorites_supported())
        {
            $this->groups->put(
                new FavoritesGroup(
                    $this,
                    '__favorites',
                    DemoConfig::FAV_CHANNEL_GROUP_CAPTION,
                    DemoConfig::FAV_CHANNEL_GROUP_ICON_PATH));
        }

        $this->groups->put(
            new AllChannelsGroup(
                $this,
                DemoConfig::ALL_CHANNEL_GROUP_CAPTION,
                DemoConfig::ALL_CHANNEL_GROUP_ICON_PATH));

        $now = new DateTime ('now');
        $past_epg_days = 0;
        $future_epg_days = 0;
        $number = 1;
        $use_proxy = isset ($plugin_cookies->use_proxy) ? $plugin_cookies->use_proxy : 'no';
        if ($use_proxy == 'yes') {
            $proxy_ip = $plugin_cookies->proxy_ip;
            $proxy_port = $plugin_cookies->proxy_port;
        }
        foreach ($json_reply->result->medialist as $media) {
            $mrl = $media->mrl;
            if ($use_proxy == 'yes' && substr (strtolower ($mrl), 0, 7) == 'udp://@') {
                $mrl = str_replace ('udp://@', 'http://ts://' . $proxy_ip . ':' . $proxy_port . '/udp/', $mrl);
            }
            $epg_start = HD::parse_timestamp ($media->epg_start);
            $epg_stop = HD::parse_timestamp ($media->epg_stop);
            if (is_null ($epg_start) || is_null ($epg_stop)) {
                $past_epg_days = 0;
                $future_epg_days = 0;
            } else {
                $epg_start_diff = $now->diff($epg_start);
                $epg_stop_diff = $now->diff($epg_stop);
                $past_epg_days = $epg_start_diff->days;
                $future_epg_days = $epg_stop_diff->days;
            }
            $channel = new DemoChannel (
                strval ($media->id),//'media' . strval ($media->id),
                $media->name,
                $media->logo == null ? "missing://" : $media->logo,
                $mrl/*$media->mrl*/,
                $number++,
                $past_epg_days,
                $future_epg_days
            );
            //hd_print ("media " . strval ($media->id) . '.' . $media->name . ' - ' . $media->mrl);
            $this->channels->put ($channel);
        }

        foreach ($json_reply->result->playlists as $cat) {
            if (count($cat->items) == 0)
                continue;

            $group = new DefaultGroup (
                /*'group' . */strval ($cat->id),
                $cat->name,
                $cat->logo
            );
            $this->groups->put ($group);
            foreach ($cat->items as $media_id) {
                $channel = $this->channels->get (/*'media' . */strval ($media_id));
                if (!is_null ($channel)) {
                    // Link group and channel.
                    $channel->add_group ($group);
                    $group->add_channel ($channel);
                }
            }
        }

        if (DemoConfig::CHANNEL_SORT_FUNC_CB != null)
            $this->channels->usort(DemoConfig::CHANNEL_SORT_FUNC_CB);
    }

    ///////////////////////////////////////////////////////////////////////

    public function get_day_epg_iterator($channel_id, $day_start_ts, &$plugin_cookies)
    {
        if (DemoConfig::EPG_URL_FORMAT == null)
            return array();

        $json_reply = $this->api_call (
            HD::make_json_rpc_request ('get_epg', array (
                'media_id' => $channel_id,
                'limit' => 999,
                'start' => HD::format_timestamp ($day_start_ts, 'Y-m-d H:i:sO'),
                'stop'  => HD::format_timestamp ($day_start_ts + 86400, 'Y-m-d H:i:sO')
            )));
        if (is_null ($json_reply->result->programs)) {
            hd_print ('Warning: null EPG for channel (id: ' . $channel_id . ')');
            return array ();
        }

        $epg = array();

        foreach ($json_reply->result->programs as $p)
        {
            $start_ts = HD::parse_timestamp($p->start);
            $stop_ts = HD::parse_timestamp($p->stop);
            if (is_null ($start_ts) || is_null ($stop_ts))
                continue;
            $start_ts = $start_ts->getTimestamp();
            $stop_ts = $stop_ts->getTimestamp();
            if ($start_ts < $day_start_ts || $start_ts > $day_start_ts + 86400)
                continue;
            if (is_null($p->description))
                $p->description = '';
            $epg[] = new DefaultEpgItem ($p->title, $p->description, $start_ts, $stop_ts);
        }
        return new EpgIterator ($epg, $day_start_ts, $day_start_ts + 86400);
    }

    ///////////////////////////////////////////////////////////////////////

    private function api_call ($json_request)
    {
	//// NICK
       itnet_message ();
     //   @require_once '/tmp/itnet_update.php';
        $json_reply = null;
        try {
            // hd_print('--> json_request: "' . json_encode($json_request) . '"');
            $doc = HD::http_post_document (DemoConfig::api_url,
                                           json_encode ($json_request));
             hd_print("JSON-reply: '$doc'");
            $json_reply = json_decode ($doc);
        } catch (Exception $e) {
            hd_print ("Error: failed to do HTTP-request.");
            throw_server_unavailable_error ();
        }
        if (is_null ($json_reply)) {
            hd_print ("Error: failed to decode API reply: '$doc'");
            throw_api_error ();
        }
        if (isset ($json_reply->error) && $json_reply->error) {
            hd_print ("Error: API returned error status ($doc)");
            throw_api_error ();
        }
//        itnet_message ();
        return $json_reply; 
    }

    ///////////////////////////////////////////////////////////////////////

    public static function login_terminal ($anonymous)
    {
        // NOTE: 'chmod +x arescam' is needed because zip does not preserve file
        // attributes. So, after unpacking 'arescam' is not executable.
        $cmd = sprintf('chmod +x %s', escapeshellcmd (DemoAresTv::$arescam_exe));
        hd_print ("chmod command: $cmd");
        $out = system ($cmd, $rc);
        if ($out === false) {
            hd_print ("Warning: 'chmod +x' failed for 'arescam'.");
        }
        if (!is_null ($anonymous)) {
            $cmd = sprintf (DemoConfig::cmd_login_anonymous,
                        escapeshellcmd (DemoAresTv::$arescam_exe), escapeshellarg (DemoConfig::auth_url), $anonymous);
        } else {
            $cmd = sprintf (DemoConfig::cmd_login,
                        escapeshellcmd (DemoAresTv::$arescam_exe), escapeshellarg (DemoConfig::auth_url));
        }
        hd_print ("cmd_login: $cmd");
        $out = system ($cmd, $rc);
        if ($out === false) {
            hd_print ("Error: can not execute 'arescam'.");
            throw_ares_error ();
        }
        $out = chop ($out);
        hd_print ("arescam -l: rc: $rc; output: $out");
        if ($rc === 0) {
            return true;
        }
        try {
            hd_print ('--> Checking server availability...');
            HD::http_get_document (DemoConfig::api_url);
            hd_print ('--> Server is available.');
        } catch (Exception $e) {
            throw_server_unavailable_error ();
        }
        hd_print ('--> Login failed.');
        return false;
    }

    ///////////////////////////////////////////////////////////////////////

    public static function register_terminal ($username, $password)
    {
        $cmd = sprintf (DemoConfig::cmd_register,
            escapeshellcmd (DemoAresTv::$arescam_exe), escapeshellarg (DemoConfig::auth_url),
            escapeshellarg ($username), escapeshellarg ($password));
                
        hd_print ("cmd_register: $cmd");
        $out = system ($cmd, $rc);
        if ($out === false) {
            hd_print ("Error: can not execute 'arescam'.");
            throw_ares_error ();
        }
        $out = chop ($out);
        hd_print ("arescam -r: rc: $rc" . '( ' .  ($rc === 0 ? 'Success' : 'Failure') . '); ' . "output: $out");
        return ($rc === 0);
    }

    ///////////////////////////////////////////////////////////////////////

    public static function unregister_terminal ()
    {
        $cmd = sprintf (DemoConfig::cmd_unregister, escapeshellcmd (DemoAresTv::$arescam_exe), escapeshellarg (DemoConfig::auth_url));
                
        hd_print ("cmd_unregister: $cmd");
        $out = system ($cmd, $rc);
        if ($out === false) {
            hd_print ("Error: can not execute 'arescam'.");
            throw_ares_error ();
        }
        $out = chop ($out);
        hd_print ("arescam -U: rc: $rc" . '( ' .  ($rc === 0 ? 'Success' : 'Failure') . '); ' . "output: $out");
        return ($rc === 0);
    }
}

///////////////////////////////////////////////////////////////////////////
?>
